@@ -7,18 +7,17 @@ import subprocess
 on_rtd = os.environ.get('READTHEDOCS', None) == 'True'
 
 if on_rtd:
-    subprocess.call('cd ..; doxygen', shell=True)
+    subprocess.check_call('cd ..; doxygen', shell=True)
 
 import sphinx_rtd_theme
 
 html_theme = "sphinx_rtd_theme"
-
 html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
 
 def setup(app):
-    app.add_stylesheet("main_stylesheet.css")
+    app.add_css_file("main_stylesheet.css")
 
-extensions = ['breathe']
+extensions = ['breathe', 'sphinx_rtd_theme']
 breathe_projects = { 'xsimd': '../xml' }
 templates_path = ['_templates']
 html_static_path = ['_static']
