@@ -78,7 +78,7 @@ namespace xsimd
         XSIMD_INLINE batch<T, A> extract_pair(batch<T, A> const& self, batch<T, A> const& other, std::size_t i, requires_arch<ssse3>) noexcept
         {
             constexpr std::size_t size = batch<T, A>::size;
-            assert(0 <= i && i < size && "index in bounds");
+            assert(i < size && "index in bounds");
             return detail::extract_pair(self, other, i, ::xsimd::detail::make_index_sequence<size>());
         }
 
