@@ -12,12 +12,25 @@
 #include "xsimd/xsimd.hpp"
 #ifndef XSIMD_NO_SUPPORTED_ARCHITECTURE
 
+#include <functional>
 #include <vector>
 
 #include "test_utils.hpp"
 
 namespace xsimd
 {
+
+    int popcount(int v)
+    {
+        // from https://graphics.stanford.edu/~seander/bithacks.html#CountBitsSetKernighan
+        int c; // c accumulates the total bits set in v
+        for (c = 0; v; c++)
+        {
+            v &= v - 1; // clear the least significant bit set
+        }
+        return c;
+    }
+
     template <class T, std::size_t N>
     struct get_bool_base
     {
@@ -61,6 +74,17 @@ namespace xsimd
     template <class T, size_t N = T::size>
     struct get_bool;
 
+    template <class T>
+    struct get_bool<batch_bool<T>, 1> : public get_bool_base<T, 1>
+    {
+        using type = batch_bool<T>;
+        type all_true = type(true);
+        type all_false = type(false);
+        type half = { 0 };
+        type ihalf = { 1 };
+        type interspersed = { 0 };
+    };
+
     template <class T>
     struct get_bool<batch_bool<T>, 2> : public get_bool_base<T, 2>
     {
@@ -131,9 +155,8 @@ namespace xsimd
 }
 
 template <class B>
-class batch_bool_test : public testing::Test
+struct batch_bool_test
 {
-protected:
     using batch_type = B;
     using value_type = typename B::value_type;
     static constexpr size_t size = B::size;
@@ -143,6 +166,7 @@ protected:
 
     array_type lhs;
     array_type rhs;
+    bool_array_type all_true;
     bool_array_type ba;
 
     batch_bool_test()
@@ -150,23 +174,71 @@ protected:
         for (size_t i = 0; i < size; ++i)
         {
             lhs[i] = value_type(i);
-            rhs[i] = i == 0 % 2 ? lhs[i] : lhs[i] * 2;
+            rhs[i] = i == 0 % 2 ? lhs[i] : lhs[i] * value_type(2);
+            all_true[i] = true;
             ba[i] = i == 0 % 2 ? true : false;
         }
     }
 
+    template <size_t... Is>
+    struct pack
+    {
+    };
+
+    template <size_t... Values>
+    void check_constructor_from_sequence(std::integral_constant<size_t, 0>, pack<Values...>) const
+    {
+        bool_array_type res = { bool(Values % 3)... };
+        bool_array_type tmp;
+        batch_bool_type b0(bool(Values % 3)...);
+        b0.store_unaligned(tmp.data());
+        INFO("batch_bool(values...)");
+        CHECK_EQ(tmp, res);
+
+        batch_bool_type b1 { bool(Values % 3)... };
+        b1.store_unaligned(tmp.data());
+        INFO("batch_bool{values...}");
+        CHECK_EQ(tmp, res);
+    }
+
+    template <size_t I, size_t... Values>
+    void check_constructor_from_sequence(std::integral_constant<size_t, I>, pack<Values...>) const
+    {
+        return check_constructor_from_sequence(std::integral_constant<size_t, I - 1>(), pack<Values..., I>());
+    }
+
+    void test_constructors() const
+    {
+        batch_bool_type a;
+        // value uninitialized, cannot test it.
+        (void)a;
+
+        bool_array_type res;
+        batch_bool_type b(true);
+        b.store_unaligned(res.data());
+        INFO("batch_bool{value}");
+        CHECK_EQ(res, all_true);
+
+        batch_bool_type c { true };
+        c.store_unaligned(res.data());
+        INFO("batch_bool{value}");
+        CHECK_EQ(res, all_true);
+
+        check_constructor_from_sequence(std::integral_constant<size_t, size>(), pack<>());
+    }
+
     void test_load_store() const
     {
         bool_array_type res;
         batch_bool_type b(batch_bool_type::load_unaligned(ba.data()));
         b.store_unaligned(res.data());
-        EXPECT_EQ(res, ba) << print_function_name("load_unaligned / store_unaligned");
+        CHECK_EQ(res, ba);
 
         alignas(xsimd::default_arch::alignment()) bool_array_type arhs(this->ba);
         alignas(xsimd::default_arch::alignment()) bool_array_type ares;
         b = batch_bool_type::load_aligned(arhs.data());
         b.store_aligned(ares.data());
-        EXPECT_EQ(ares, arhs) << print_function_name("load_aligned / store_aligned");
+        CHECK_EQ(ares, arhs);
     }
 
     void test_any_all() const
@@ -176,33 +248,33 @@ protected:
         {
             auto any_check_false = (batch_lhs() != batch_lhs());
             bool any_res_false = xsimd::any(any_check_false);
-            EXPECT_FALSE(any_res_false) << print_function_name("any (false)");
+            CHECK_FALSE(any_res_false);
             auto any_check_true = (batch_lhs() == batch_rhs());
             bool any_res_true = xsimd::any(any_check_true);
-            EXPECT_TRUE(any_res_true) << print_function_name("any (true)");
+            CHECK_UNARY(any_res_true);
 
             for (const auto& vec : bool_g.almost_all_false())
             {
                 batch_bool_type b = batch_bool_type::load_unaligned(vec.data());
                 bool any_res = xsimd::any(b);
-                EXPECT_TRUE(any_res) << print_function_name("any (almost_all_false)");
+                CHECK_UNARY(any_res);
             }
 
             for (const auto& vec : bool_g.almost_all_true())
             {
                 batch_bool_type b = batch_bool_type::load_unaligned(vec.data());
                 bool any_res = xsimd::any(b);
-                EXPECT_TRUE(any_res) << print_function_name("any (almost_all_true)");
+                CHECK_UNARY(any_res);
             }
         }
         // all
         {
             auto all_check_false = (batch_lhs() == batch_rhs());
             bool all_res_false = xsimd::all(all_check_false);
-            EXPECT_FALSE(all_res_false) << print_function_name("all (false)");
+            CHECK_FALSE(all_res_false);
             auto all_check_true = (batch_lhs() == batch_lhs());
             bool all_res_true = xsimd::all(all_check_true);
-            EXPECT_TRUE(all_res_true) << print_function_name("all (true)");
+            CHECK_UNARY(all_res_true);
 
             for (const auto& vec : bool_g.almost_all_false())
             {
@@ -211,37 +283,37 @@ protected:
                 // give expected result
                 batch_bool_type b = batch_bool_type::load_unaligned(vec.data());
                 bool all_res = xsimd::all(b);
-                EXPECT_FALSE(all_res) << print_function_name("all (almost_all_false)");
+                CHECK_FALSE(all_res);
             }
 
             for (const auto& vec : bool_g.almost_all_true())
             {
                 batch_bool_type b = batch_bool_type::load_unaligned(vec.data());
                 bool all_res = xsimd::all(b);
-                EXPECT_FALSE(all_res) << print_function_name("all (almost_all_true)");
+                CHECK_FALSE(all_res);
             }
         }
         // none
         {
             auto none_check_false = (batch_lhs() == batch_rhs());
             bool none_res_false = xsimd::none(none_check_false);
-            EXPECT_FALSE(none_res_false) << print_function_name("none (false)");
+            CHECK_FALSE(none_res_false);
             auto none_check_true = (batch_lhs() != batch_lhs());
             bool none_res_true = xsimd::none(none_check_true);
-            EXPECT_TRUE(none_res_true) << print_function_name("none (true)");
+            CHECK_UNARY(none_res_true);
 
             for (const auto& vec : bool_g.almost_all_false())
             {
                 batch_bool_type b = batch_bool_type::load_unaligned(vec.data());
                 bool none_res = xsimd::none(b);
-                EXPECT_FALSE(none_res) << print_function_name("none (almost_all_false)");
+                CHECK_FALSE(none_res);
             }
 
             for (const auto& vec : bool_g.almost_all_true())
             {
                 batch_bool_type b = batch_bool_type::load_unaligned(vec.data());
                 bool none_res = xsimd::none(b);
-                EXPECT_FALSE(none_res) << print_function_name("none (almost_all_true)");
+                CHECK_FALSE(none_res);
             }
         }
     }
@@ -253,12 +325,11 @@ protected:
         // operator!=
         {
             bool res = xsimd::all(bool_g.half != bool_g.ihalf);
-            EXPECT_TRUE(res) << print_function_name("operator!=");
+            CHECK_UNARY(res);
         }
         // operator==
         {
-            bool res = xsimd::all(bool_g.half == !bool_g.ihalf);
-            EXPECT_TRUE(res) << print_function_name("operator==");
+            CHECK_BATCH_EQ(bool_g.half, !bool_g.ihalf);
         }
         // operator &&
         {
@@ -266,7 +337,7 @@ protected:
             bool_array_type ares;
             res.store_unaligned(ares.data());
             size_t nb_false = std::count(ares.cbegin(), ares.cend(), false);
-            EXPECT_EQ(nb_false, s) << print_function_name("operator&&");
+            CHECK_EQ(nb_false, s);
         }
         // operator ||
         {
@@ -274,7 +345,7 @@ protected:
             bool_array_type ares;
             res.store_unaligned(ares.data());
             size_t nb_true = std::count(ares.cbegin(), ares.cend(), true);
-            EXPECT_EQ(nb_true, s) << print_function_name("operator||");
+            CHECK_EQ(nb_true, s);
         }
         // operator ^
         {
@@ -282,7 +353,7 @@ protected:
             bool_array_type ares;
             res.store_unaligned(ares.data());
             size_t nb_true = std::count(ares.cbegin(), ares.cend(), true);
-            EXPECT_EQ(nb_true, s) << print_function_name("operator^");
+            CHECK_EQ(nb_true, s);
         }
         // bitwise_andnot
         {
@@ -290,27 +361,105 @@ protected:
             bool_array_type ares;
             res.store_unaligned(ares.data());
             size_t nb_false = std::count(ares.cbegin(), ares.cend(), false);
-            EXPECT_EQ(nb_false, s) << print_function_name("bitwise_andnot");
+            CHECK_EQ(nb_false, s);
         }
     }
 
     void test_bitwise_operations() const
     {
         auto bool_g = xsimd::get_bool<batch_bool_type> {};
-        // operator~
+        // operator version
         {
-            bool res = xsimd::all(bool_g.half == ~bool_g.ihalf);
-            EXPECT_TRUE(res) << print_function_name("operator~");
+            INFO("operator~");
+            CHECK_BATCH_EQ(bool_g.half, ~bool_g.ihalf);
         }
-        // operator|
         {
-            bool res = xsimd::all((bool_g.half | bool_g.ihalf) == bool_g.all_true);
-            EXPECT_TRUE(res) << print_function_name("operator|");
+            INFO("operator|");
+            CHECK_BATCH_EQ(bool_g.half | bool_g.ihalf, bool_g.all_true);
         }
-        // operator&
         {
-            bool res = xsimd::all((bool_g.half & bool_g.ihalf) == bool_g.all_false);
-            EXPECT_TRUE(res) << print_function_name("operator&");
+            INFO("operator&");
+            CHECK_BATCH_EQ(bool_g.half & bool_g.ihalf, bool_g.all_false);
+        }
+        // free function version
+        {
+            INFO("bitwise_not");
+            CHECK_BATCH_EQ(bool_g.half, xsimd::bitwise_not(bool_g.ihalf));
+        }
+        {
+            INFO("bitwise_or");
+            CHECK_BATCH_EQ(xsimd::bitwise_or(bool_g.half, bool_g.ihalf), bool_g.all_true);
+        }
+        {
+            INFO("bitwise_and");
+            CHECK_BATCH_EQ(xsimd::bitwise_and(bool_g.half, bool_g.ihalf), bool_g.all_false);
+        }
+    }
+
+    void test_update_operations() const
+    {
+        auto bool_g = xsimd::get_bool<batch_bool_type> {};
+        {
+            auto tmp = bool_g.half;
+            tmp |= bool_g.ihalf;
+            bool res = xsimd::all(tmp);
+            INFO("operator|=");
+            CHECK_UNARY(res);
+        }
+        {
+            auto tmp = bool_g.half;
+            tmp &= bool_g.half;
+            INFO("operator&=");
+            CHECK_BATCH_EQ(tmp, bool_g.half);
+        }
+        {
+            auto tmp = bool_g.half;
+            tmp ^= bool_g.ihalf;
+            bool res = xsimd::all(tmp);
+            INFO("operator^=");
+            CHECK_UNARY(res);
+        }
+    }
+
+    void test_mask() const
+    {
+        auto bool_g = xsimd::get_bool<batch_bool_type> {};
+        const uint64_t full_mask = ((uint64_t)-1) >> (64 - batch_bool_type::size);
+        CHECK_EQ(bool_g.all_false.mask(), 0);
+        CHECK_EQ(batch_bool_type::from_mask(bool_g.all_false.mask()).mask(), bool_g.all_false.mask());
+
+        CHECK_EQ(bool_g.all_true.mask(), full_mask);
+        CHECK_EQ(batch_bool_type::from_mask(bool_g.all_true.mask()).mask(), bool_g.all_true.mask());
+
+        CHECK_EQ(bool_g.half.mask(), full_mask & ((uint64_t)-1) << (batch_bool_type::size / 2));
+        CHECK_EQ(batch_bool_type::from_mask(bool_g.half.mask()).mask(), bool_g.half.mask());
+
+        CHECK_EQ(bool_g.ihalf.mask(), full_mask & ~(((uint64_t)-1) << (batch_bool_type::size / 2)));
+        CHECK_EQ(batch_bool_type::from_mask(bool_g.ihalf.mask()).mask(), bool_g.ihalf.mask());
+
+        CHECK_EQ(bool_g.interspersed.mask(), full_mask & 0xAAAAAAAAAAAAAAAAul);
+        CHECK_EQ(batch_bool_type::from_mask(bool_g.interspersed.mask()).mask(), bool_g.interspersed.mask());
+    }
+
+    void test_count() const
+    {
+        auto bool_g = xsimd::get_bool<batch_bool_type> {};
+        CHECK_EQ(count(bool_g.all_false), 0);
+        CHECK_EQ(count(bool_g.all_true), batch_bool_type::size);
+        CHECK_EQ(count(bool_g.half), batch_bool_type::size / 2);
+    }
+
+    void test_comparison() const
+    {
+        auto bool_g = xsimd::get_bool<batch_bool_type> {};
+        // eq
+        {
+            CHECK_BATCH_EQ(bool_g.half, !bool_g.ihalf);
+            CHECK_BATCH_EQ(xsimd::eq(bool_g.half, !bool_g.ihalf), bool_g.all_true);
+        }
+        // neq
+        {
+            CHECK_BATCH_EQ(xsimd::neq(bool_g.half, bool_g.ihalf), bool_g.all_true);
         }
     }
 
@@ -326,25 +475,26 @@ private:
     }
 };
 
-TYPED_TEST_SUITE(batch_bool_test, batch_types, simd_test_names);
-
-TYPED_TEST(batch_bool_test, load_store)
+TEST_CASE_TEMPLATE("[xsimd batch bool]", B, BATCH_TYPES)
 {
-    this->test_load_store();
-}
+    batch_bool_test<B> Test;
 
-TYPED_TEST(batch_bool_test, any_all)
-{
-    this->test_any_all();
-}
+    SUBCASE("constructors") { Test.test_constructors(); }
 
-TYPED_TEST(batch_bool_test, logical_operations)
-{
-    this->test_logical_operations();
-}
+    SUBCASE("load store") { Test.test_load_store(); }
 
-TYPED_TEST(batch_bool_test, bitwise_operations)
-{
-    this->test_bitwise_operations();
+    SUBCASE("any all") { Test.test_any_all(); }
+
+    SUBCASE("logical operations") { Test.test_logical_operations(); }
+
+    SUBCASE("bitwise operations") { Test.test_bitwise_operations(); }
+
+    SUBCASE("update operations") { Test.test_update_operations(); }
+
+    SUBCASE("mask") { Test.test_mask(); }
+
+    SUBCASE("count") { Test.test_count(); }
+
+    SUBCASE("eq neq") { Test.test_comparison(); }
 }
 #endif
